#!/usr/bin/env python

class Orchestra(object):
    """Encapsulates the orchestra"""

    def __init__(self, sr = 44100, ksmps = 10, nchnls = 2, name = '', author = '', comment = ''):
        if sr % ksmps != 0:
            raise ValueError, "control rate must result an integer value. got sr=%d and ksmps=%d" % (sr, ksmps)
        
        self.sr = sr
        self.ksmps = ksmps
        self.nchnls = nchnls

        self.name = name
        self.author = author
        self.comment = comment

        self.instrumentList = []
        self.globalList = []

    def addInstrument(self, instrument):
        """Add an instrument to our list."""

        if not instrument in self.instrumentList:
            self.instrumentList.append(instrument)

    def addGlobal(self, globalObject):
        """Add a global value to our orchestra."""

        if not globalObject in self.globalList:
            self.globalList.append(globalObject)

    def __generatePreamble(self):
        """Generate the preamble for the orchestra."""

        # create the comments section
        preamble = self.comment + "\n"
        preamble = preamble + "; %s\n" % self.name
        preamble = preamble + "; %s\n" % self.author

        # create the header
        preamble = preamble + "\n"
        preamble = preamble + "sr \t= \t%d\n" % self.sr
        preamble = preamble + "kr \t= \t%d\n" % (self.sr/self.ksmps)
        preamble = preamble + "ksmps \t= \t%d\n" % self.ksmps
        preamble = preamble + "nchnls \t= \t%d\n" % self.nchnls

        self.preamble = preamble
        return preamble

    def __generateGlobals(self):
        """Generate the globals, if any."""

        globalVars = ""
        for globalVar in self.globalList:
            globalVars = globalVars + globalVar.globalVar

        self.globalVars = globalVars
        return globalVars

    def __generateInstruments(self):
        """Generate the instrument list."""

        instruments = ""
        for instrument in self.instrumentList:
            instruments = instruments + instrument.definition
            instruments = instruments + "\n\n"

        self.instruments = instruments 
        return instruments

    def generateOrchestra(self):
        """Generate the orchestra section."""

        orchestra = "<CsInstruments>\n"

        orchestra = orchestra + self.__generatePreamble()
        orchestra = orchestra + "\n\n"

        orchestra = orchestra + "; Globals\n"
        orchestra = orchestra + self.__generateGlobals()
        orchestra = orchestra + "\n\n"
        
        orchestra = orchestra + "; Instruments\n"
        orchestra = orchestra + self.__generateInstruments()

        orchestra = orchestra + "</CsInstruments>"

        self.orchestra = orchestra

        return orchestra

class Instrument(object):
    """A particular instrument object."""

    def __init__(self, number = None, definition = ''):
        if number is not None:
            self.number = number

        self.definition = definition

    def setDefinition(self, definition):
        if definition:
            logging.warn("Instrument definition already set, overwritting...")

        self.definition = definition

class Global(object):
    """Global varaibles for csound."""

    def __init__(self, globalVar = ''):
        self.globalVar = globalVar

    def setGlobalVar(self, globalVar):
        """Sets the global variable."""

        self.globalVar = globalVar
